@@ -1,14 +1,16 @@
-#$Id: clihub.pm 472 2009-08-25 05:52:44Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/clihub.pm $
-package Net::DirectConnect::clihub;
+#$Id: clihub.pm 540 2010-01-11 03:04:20Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/clihub.pm $
+package    #hide from cpan
+  Net::DirectConnect::clihub;
+use strict;
 use Time::HiRes qw(time sleep);
 use Data::Dumper;    #dev only
 $Data::Dumper::Sortkeys = 1;
+$Data::Dumper::Indent   = 1;
 use Net::DirectConnect;
 use Net::DirectConnect::clicli;
 #use Net::DirectConnect::http;
-use strict;
 no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 472 $' ) )[1];
+our $VERSION = ( split( ' ', '$Revision: 540 $' ) )[1];
 use base 'Net::DirectConnect';
 
 sub init {
@@ -36,70 +38,82 @@ sub init {
     'search_every'     => 10,
     'search_every_min' => 10,
     'auto_connect'     => 1,
-    'NoGetINFO'        => 1,          #test
-    'NoHello'          => 1,
-    'UserIP2'          => 1,
-    'Version'          => '1,0091',
-    'auto_GetNickList' => 1,
-    'follow_forcemove' => 1,
+    'NoGetINFO'        => 1,                                                                                            #test
+    'NoHello'          => 1, 'UserIP2' => 1, 'Version' => '1,0091', 'auto_GetNickList' => 1, 'follow_forcemove' => 1,
+    #ADC
+    #'connect_protocol' => 'ADC/0.10',
+    #'message_type'     => 'H',
     @_,
     'incomingclass' => 'Net::DirectConnect::clicli',
     'periodic'      => sub { $self->cmd( 'search_buffer', ) if $self->{'socket'}; },
   );
+  #$self->log($self, 'inited',"MT:$self->{'message_type'}", ' with', Dumper  \@_);
   $self->baseinit();
+  #$self->log( $self, 'inited3', "MT:$self->{'message_type'}", ' with' );
+  #You are already in the hub.
   $self->{'parse'} ||= {
     'chatline' => sub {
       my ( $nick, $text ) = $_[0] =~ /^(?:<|\* )(.+?)>? (.+)$/s;
       #$self->log('dcdev', 'chatline parse', Dumper(\@_,$nick, $text));
       $self->log( 'warn', "[$nick] oper: already in the hub [$self->{'Nick'}]" ), $self->cmd('nick_generate'),
         $self->reconnect(),
-        if ( ( !keys %{ $self->{'NickList'} } or $self->{'NickList'}->{$nick}{'oper'} )
+        if ( ( !keys %{ $self->{'NickList'} } or $self->{'NickList'}{$nick}{'oper'} )
         and $text eq 'You are already in the hub.' );
-      if ( $self->{'NickList'}->{$nick}{'oper'} or $nick eq 'Hub-Security' ) {
+      if ( $self->{'NickList'}{$nick}{'oper'} or $self->{'NickList'}{$nick}{'hubbot'} or $nick eq 'Hub-Security' ) {
         if (
              $text =~ /^(?:Minimum search interval is|  ):(\d+)s/
           or $text =~ /Search ignored\.  Please leave at least (\d+) seconds between search attempts\./  #Hub-Security opendchub
           )
         {
-          $self->log( 'warn', "[$nick] oper: set min interval = $1" );
-          $self->{'search_every'} = int $1 || $self->{'search_every_min'};
+          $self->{'search_every'} = int( rand(5) + $1 || $self->{'search_every_min'} );
+          $self->log( 'warn', "[$nick] oper: set min interval = $self->{'search_every'}" );
           $self->search_retry();
         }
-        if ( $text =~ /  (\d+)    \./
-          or $text eq '     !' )
+        if ( $text =~ /(?: )? (\d+)    \./i
+          or $text =~ /(?:Please )?wait (\d+) seconds before next search\./i
+          or $text eq '     !'
+          or $text eq "Please don't flood with searches!" )
         {
-          $self->log( 'warn', "[$nick] oper: increase min interval +=", int $1 || $self->{'search_every_min'} ),
-            $self->{'search_every'} += int $1 || $self->{'search_every_min'};
+          $self->{'search_every'} += int( rand(5) + $1 || $self->{'search_every_min'} );
+          $self->log( 'warn', "[$nick] oper: increase min interval => $self->{'search_every'}" );
           $self->search_retry();
         }
       }
+      if ( !$self->{count_parse}{chatline} and $text =~ /PtokaX/i ) {
+        #$self->log( 'dev', "[$nick] - probably hub bot" );
+        $self->{'NickList'}{$nick}{'hubbot'} = 1;
+      }
       $self->search_retry(),
         if $self->{'NickList'}->{$nick}{'oper'} and $text eq 'Sorry Hub is busy now, no search, try later..';
     },
     'welcome' => sub {
       my ( $nick, $text ) = $_[0] =~ /^(?:<|\* )(.+?)>? (.+)$/s;
-      if ( ( !keys %{ $self->{'NickList'} } or !exists $self->{'NickList'}->{$nick} or $self->{'NickList'}->{$nick}{'oper'} )
-        and $text =~ /^Bad nickname: unallowed characters, use these (\S+)/ )
-      {
-        my $try = $self->{'Nick'};
-        $try =~ s/[^\Q$1\E]//g;
-        $self->log( 'warn', "CHNICK $self->{'Nick'} -> $try" );
-        $self->{'Nick'} = $try if length $try;
+      if ( !keys %{ $self->{'NickList'} } or !exists $self->{'NickList'}->{$nick} or $self->{'NickList'}->{$nick}{'oper'} ) {
+        if ( $text =~ /^Bad nickname: unallowed characters, use these (\S+)/ )
+          #
+        {
+          my $try = $self->{'Nick'};
+          $try =~ s/[^\Q$1\E]//g;
+          $self->log( 'warn', "CHNICK $self->{'Nick'} -> $try" );
+          $self->{'Nick'} = $try if length $try;
+        } elsif ( $text =~ /Bad nickname: Wait (\d+)sec before reconnecting/i ) {
+          sleep $1 + 1;
+        }
       }
     },
     'Lock' => sub {
-      $self->log( "lockparse", @_ );
+      #$self->log( "lockparse", @_ );
       $self->{'sendbuf'} = 1;
       $self->cmd('Supports');
       $_[0] =~ /^(.+?)(\s+Pk=.+)?\s*$/is;
-      print "lock[$1]\n";
+      #print "lock[$1]\n";
       $self->cmd( 'Key', $self->lock2key($1) );
       $self->{'sendbuf'} = 0;
       $self->cmd('ValidateNick');
     },
     'Hello' => sub {
+      #$self->log('info', "HELLO recieved, connected. me=[$self->{'Nick'}]", @_);
       return unless $_[0] eq $self->{'Nick'};
-      #$self->log('info', "HELLO recieved, connected.");
       $self->{'sendbuf'} = 1;
       $self->cmd('Version');
       $self->{'sendbuf'} = 0 unless $self->{'auto_GetNickList'};
@@ -112,6 +126,7 @@ sub init {
       $self->supports_parse( $_[0], $self );
     },
     'ValidateDenide' => sub {
+      $self->log( 'warn', "ValidateDenide", $self->{'Nick'}, @_ );
       $self->cmd('nick_generate');
       $self->cmd('ValidateNick');
     },
@@ -157,13 +172,14 @@ sub init {
       return if $self->{'clients'}{ $host . ':' . $port }->{'socket'};
       $self->{'clients'}{ $host . ':' . $port } = Net::DirectConnect::clicli->new(
         %$self, $self->clear(),
-        'host'         => $host,
-        'port'         => $port,
-        'want'         => \%{ $self->{'want'} },
-        'NickList'     => \%{ $self->{'NickList'} },
-        'IpList'       => \%{ $self->{'IpList'} },
-        'PortList'     => \%{ $self->{'PortList'} },
-        'handler'      => \%{ $self->{'handler'} },
+        'host' => $host,
+        'port' => $port,
+#'want'         => \%{ $self->{'want'} },        'NickList'     => \%{ $self->{'NickList'} },        'IpList'       => \%{ $self->{'IpList'} },        'PortList'     => \%{ $self->{'PortList'} },        'handler'      => \%{ $self->{'handler'} },
+        'want'         => $self->{'want'},
+        'NickList'     => $self->{'NickList'},
+        'IpList'       => $self->{'IpList'},
+        'PortList'     => $self->{'PortList'},
+        'handler'      => $self->{'handler'},
         'auto_connect' => 1,
       );
     },
@@ -181,7 +197,7 @@ sub init {
     'Search' => sub {
       my $search = $_[0];
       $self->cmd('make_hub');
-      my %s = ( 'time' => int( time() ), 'hub' => $self->{'hub'}, );
+      my %s = ( 'time' => int( time() ), 'hub' => $self->{'hub_name'}, );
       ( $s{'who'}, $s{'cmds'} ) = split /\s+/, $search;
       $s{'cmd'} = [ split /\?/, $s{'cmds'} ];
       if ( $s{'who'} =~ /^Hub:(.+)$/ ) { $s{'nick'} = $1; }
@@ -189,11 +205,63 @@ sub init {
       if   ( $s{'cmd'}[4] =~ /^TTH:([0-9A-Z]{39})$/ ) { $s{'tth'}    = $1; }
       else                                            { $s{'string'} = $s{'cmd'}[4]; }
       $s{'string'} =~ tr/$/ /;
+      $self->cmd('make_hub');
+
+      if ( $self->{'share_tth'} and $s{'tth'} and $self->{'share_tth'}{ $s{'tth'} } ) {
+        $self->log(
+          'adcdev', 'Search', $s{'who'},
+          $self->{'share_tth'}{ $s{'tth'} },
+          -s $self->{'share_tth'}{ $s{'tth'} },
+          -e $self->{'share_tth'}{ $s{'tth'} }
+          ),
+          $self->{'share_tth'}{ $s{'tth'} } =~ tr{\\}{/};
+        $self->{'share_tth'}{ $s{'tth'} } =~ s{^/+}{};
+        my $path;
+        if ( $self->{'adc'} ) { $path = $self->adc_path_encode( $self->{'share_tth'}{ $s{'tth'} } ); }
+        else {
+          $path = $self->{'share_tth'}{ $s{'tth'} };
+          $path =~ s{^\w:}{};
+          $path =~ s{^\W+}{};
+          $path =~ tr{/}{\\};
+        }
+        local @_ = (
+          'SR', (
+            #( $self->{'M'} eq 'P' or !$self->{'myport_tcp'} or !$self->{'myip'} )            ?
+            $self->{'Nick'}
+              #: $self->{'myip'} . ':' . $self->{'myport_tcp'}
+          ),
+          $path . "\x05" . ( -s $self->{'share_tth'}{ $s{'tth'} } or -1 ),
+          $self->{'S'} . '/'
+            . $self->{'S'} . "\x05" . "TTH:"
+            . $s{'tth'}
+            #. ( $self->{'M'} eq 'P' ? " ($self->{'host'}:$self->{'port'})" : '' ),
+            #. (  " ($self->{'host'}:$self->{'port'})\x05$s{'nick'}"  ),
+            . ( " ($self->{'host'}:$self->{'port'})" . ( ( $s{'ip'} and $s{'port'} ) ? '' : "\x05$s{'nick'}" ) ),
+#. ( $self->{'M'} eq 'P' ? " ($self->{'host'}:$self->{'port'})\x05$s{'nick'}" : '' ),
+#{ SI => -s $self->{'share_tth'}{ $params->{TR} },SL => $self->{INF}{SL},FN => $self->adc_path_encode( $self->{'share_tth'}{ $params->{TR} } ),=> $params->{TO} || $self->make_token($peerid),TR => $params->{TR}}
+        );
+        if ( $s{'ip'} and $s{'port'} ) { $self->send_udp( $s{'ip'}, $s{'port'}, join ' ', @_ ); }
+        else                           { $self->cmd(@_); }
+      }
+#'SR', ( $self->{'M'} eq 'P' ? "Hub:$self->{'Nick'}" : "$self->{'myip'}:$self->{'myport_udp'}" ),        join '?',
+#Hub:	[Outgoing][80.240.208.42:4111]	 	$SR prrrrroo0 distr\s60\games\10598_paintball2.zip621237 1/2TTH:3TFVOXE2DS6W62RWL2QBEKZBQLK3WRSLG556ZCA (80.240.208.42:4111)breathe|
+#$SR prrrrroo0 distr\moscow\mom\Mo\P\Paintball.htm1506 1/2TTH:NRRZNA5MYJSZGMPQ634CPGCPX3ZBRLKHAACPAFQ (80.240.208.42:4111)breathe|
+#$SR prrrrroo0 distr\moscow\mom\Map\P\Paintball.htm3966 1/2TTH:QLRRMET6MSNJTIRKBDLQYU6RMI5QVZDZOGAXEXA (80.240.208.42:4111)breathe|
+#$SR ILICH _07_2007\bases\sidhouse.DBF120923801 6/8TTH:4BAKR7LLXE65I6S4HASIXWIZONBEFS7VVZ7QQ2Y (80.240.211.183:411)
+#$SR gellarion7119 MuZonnO\Mark Knopfler - Get Lucky (2009)\mark_knopfler_-_you_cant_beat_the_house.mp36599140 7/7TTH:IDPHZ4AJIIWDYOFEKCCVJUNVIPGSGTYFW5CGEQQ (80.240.211.183:411)
+#$SR 13th_day \ \sacrifice_penthouse02.jpg62412 0/20TTH:GHMWHVBKRLF52V26VFO4M4RUQ65NC3YKWIW7FPI (80.240.211.183:411)
+#DIRECT:
+#$SR server1 server\Unsorted\Desperate.Housewives.S04.720p.HDTV.x264\desperate.housewives.s04e03.720p.hdtv.x264.Rus.Eng.mkv1194423977 2/2TTH:6YWRGDXNQJEOGSB4Q7Y3Y7XRM7EXPLUK7GBRJ3A (80.240.211.183:411)
+#$SR MikMEBX Deep purple\1980-1988\08-The House Of Blue Light.1987 10/10[ f12p.ru ][ F12P-HUB ] -  ...      ..      (80.240.211.183)
+#PASSIVE
+#$SR ILICH _07_2007\bases\sidhouse.DBF120923801 6/8TTH:4BAKR7LLXE65I6S4HASIXWIZONBEFS7VVZ7QQ2Y (80.240.211.183:411)
+#$SR gellarion7119 MuZonnO\Mark Knopfler - Get Lucky (2009)\mark_knopfler_-_you_cant_beat_the_house.mp36599140 7/7TTH:IDPHZ4AJIIWDYOFEKCCVJUNVIPGSGTYFW5CGEQQ (80.240.211.183:411)
+#$SR SALAGA \\XXX\xxx Penthouse.avi732665856 0/5TTH:3OFCM6GPQZNBNAMV6SRDFHFPK2X76EO6UCIO7ZQ (80.240.211.183:411)
       return \%s;
     },
     'SR' => sub {
       $self->cmd('make_hub');
-      my %s = ( 'time' => int( time() ), 'hub' => $self->{'hub'}, );
+      my %s = ( 'time' => int( time() ), 'hub' => $self->{'hub_name'}, );
       ( $s{'nick'}, $s{'str'} ) = split / /, $_[0], 2;
       $s{'str'} = [ split /\x05/, $s{'str'} ];
       $s{'file'} = shift @{ $s{'str'} };
@@ -215,17 +283,22 @@ sub init {
     },
     'UserCommand' => sub {
     },
-    #
-    #=================
-    #ADC dev
-    #
-    'ISUP' => sub { }, 'ISID' => sub { $self->{'sid'} = $_[0] }, 'IINF' => sub { $self->cmd('BINF') },
-    #todo
-    'IQUI' => sub { },
-    'ISTA' => sub { $self->log( 'dcerr', @_ ) },
   };
+
+=COMMANDS
+
+
+
+
+
+
+
+
+=cut  
+
   $self->{'cmd'} = {
     'chatline' => sub {
+      my $self = shift if ref $_[0];
       for (@_) {
         if ( $self->{'min_chat_delay'} and ( time - $self->{'last_chat_time'} < $self->{'min_chat_delay'} ) ) {
           $self->log( 'dbg', 'sleep', $self->{'min_chat_delay'} - time + $self->{'last_chat_time'} );
@@ -236,11 +309,12 @@ sub init {
           'dcdmp',
           "($self->{'number'}) we send [",
           "<$self->{'Nick'}> $_|",
-          "]:", $self->{'socket'}->send("<$self->{'Nick'}> $_|"), $!
+          "]:", $self->send("<$self->{'Nick'}> $_|"), $!
         );
       }
     },
     'To' => sub {
+      my $self = shift if ref $_[0];
       my $to = shift;
       $self->sendcmd( 'To:', $to, "From: $self->{'Nick'} \$<$self->{'Nick'}> $_" ) for (@_);
     },
@@ -249,12 +323,15 @@ sub init {
       $self->sendcmd( 'Key', $_[0] );
     },
     'ValidateNick' => sub {
+      my $self = shift if ref $_[0];
       $self->sendcmd( 'ValidateNick', $self->{'Nick'} );
     },
     'Version' => sub {
+      my $self = shift if ref $_[0];
       $self->sendcmd( 'Version', $self->{'Version'} );
     },
     'MyINFO' => sub {
+      my $self = shift if ref $_[0];
       $self->sendcmd( 'MyINFO', '$ALL', $self->myinfo() );
     },
     'GetNickList' => sub {
@@ -288,101 +365,53 @@ sub init {
       $self->sendcmd( 'Supports', $self->supports() || return );
     },
     'Quit' => sub {
+      my $self = shift if ref $_[0];
       $self->sendcmd( 'Quit', $self->{'Nick'} );
       $self->disconnect();
     },
+    'SR' => sub {
+      my $self = shift if ref $_[0];
+      $self->sendcmd( 'SR', @_ );
+    },
     'Search' => sub {
       my $self = shift if ref $_[0];
       $self->sendcmd( 'Search', ( $self->{'M'} eq 'P' ? "Hub:$self->{'Nick'}" : "$self->{'myip'}:$self->{'myport_udp'}" ),
         join '?', @_ );
     },
-    'search_buffer' => sub {
-      my $self = shift if ref $_[0];
-      push( @{ $self->{'search_todo'} }, [@_] ) if @_;
-      return unless @{ $self->{'search_todo'} || return };
-#$self->log( 'dcdev', "search too fast [$self->{'search_every'}], len=", scalar @{ $self->{'search_todo'} } )        if @_ and scalar @{ $self->{'search_todo'} } > 1;
-      return if time() - $self->{'search_last_time'} < $self->{'search_every'} + 2;
-      $self->{'search_last'} = shift( @{ $self->{'search_todo'} } );
-      $self->{'search_todo'} = undef unless @{ $self->{'search_todo'} };
-      $self->sendcmd( 'Search', $self->{'M'} eq 'P' ? 'Hub:' . $self->{'Nick'} : "$self->{'myip'}:$self->{'myport'}",
-        join '?', @{ $self->{'search_last'} } );
-      $self->{'search_last_time'} = time();
-    },
     'search_tth' => sub {
       my $self = shift if ref $_[0];
       $self->{'search_last_string'} = undef;
-      $self->cmd( 'search_buffer', 'F', 'T', '0', '9', 'TTH:' . $_[0] );
+      if ( $self->{'adc'} ) {
+        #$self->cmd( 'search_buffer', { TO => $self->make_token(), TR => $_[0], } );
+      }    #toauto
+      else { $self->cmd( 'search_buffer', 'F', 'T', '0', '9', 'TTH:' . $_[0] ); }
     },
     'search_string' => sub {
       my $self = shift if ref $_[0];
       my $string = $_[0];
-      $self->{'search_last_string'} = $string;
-      $string =~ tr/ /$/;
-      $self->cmd( 'search_buffer', 'F', 'T', '0', '1', $string );
+      if ( $self->{'adc'} ) {
+        #$self->cmd( 'search_buffer', { TO => 'auto', map AN => $_, split /\s+/, $string } );
+        #$self->cmd( 'search_buffer', ( map { 'AN' . $_ } split /\s+/, $string ), { TO => $self->make_token(), } );    #TOauto
+      } else {
+        $self->{'search_last_string'} = $string;
+        $string =~ tr/ /$/;
+        $self->cmd( 'search_buffer', 'F', 'T', '0', '1', $string );
+      }
     },
-    'search' => sub {
+    'search_send' => sub {
       my $self = shift if ref $_[0];
-      return $self->cmd( 'search_tth', @_ ) if length $_[0] == 39 and $_[0] =~ /^[0-9A-Z]+$/;
-      return $self->cmd( 'search_string', @_ ) if length $_[0];
-    },
-    'search_retry' => sub {
-      my $self = shift if ref $_[0];
-      unshift( @{ $self->{'search_todo'} }, $self->{'search_last'} ) if ref $self->{'search_last'} eq 'ARRAY';
-      $self->{'search_last'} = undef;
+      $self->sendcmd( 'Search',
+        ( ( $self->{'myip'} && $self->{'myport_udp'} ) ? "$self->{'myip'}:$self->{'myport_udp'}" : 'Hub:' . $self->{'Nick'} ),
+        join '?', @{ $_[0] || $self->{'search_last'} } );
     },
     'make_hub' => sub {
-      $self->{'hub'} ||= $self->{'host'} . ( ( $self->{'port'} and $self->{'port'} != 411 ) ? ':' . $self->{'port'} : '' );
-    },
-    'nick_generate' => sub {
-      $self->{'nick_base'} ||= $self->{'Nick'};
-      $self->{'Nick'} = $self->{'nick_base'} . int( rand( $self->{'nick_random'} || 100 ) );
+      my $self = shift if ref $_[0];
+      $self->{'hub_name'} ||= $self->{'host'} . ( ( $self->{'port'} and $self->{'port'} != 411 ) ? ':' . $self->{'port'} : '' );
     },
     #
-    #=================
-    #ADC dev
-    #
-    'connect_aft' => sub { $self->cmd('HSUP') if $self->{'protocol'} eq 'adc' },
-    'HSUP' => sub {
-      $self->{'SUPADS'} ||= [qw(BAS0 BASE TIGR UCM0 BLO0)];
-      $self->{'SUPAD'} ||= { map { $_ => 1 } @{ $self->{'SUPADS'} } };
-      $self->sendcmd( 'HSUP', ( map { 'AD' . $_ } @{ $self->{'SUPADS'} } ), ( map { 'RM' . $_ } keys %{ $self->{'SUPRM'} } ), );
-      #ADBAS0 ADBASE ADTIGR ADUCM0 ADBLO0
-    },
-    'BINF' => sub {
-      $self->{'BINFS'} ||= [qw(ID PD NI SL SS SF HN HR HO VE US SU)];
-      #$self->{'ID'} ||= 'FXC3WTTDXHP7PLCCGZ6ZKBHRVAKBQ4KUINROXXI';
-      #$self->{'PD'} ||='P26YAWX3HUNSTEXXYRGOIAAM2ZPMLD44HCWQEDY';
-      $self->{'NI'} ||= $self->{'Nick'} || 'perlAdcDev';
-      #hash() returns a 192 bit hash
-      #$self->log('TIGERTE',  Digest::Tiger::hash('Tiger'));
-      #$self->log('TIGERTEST',  MIME::Base32::encode(Digest::Tiger::hash('Tiger')));
-      sub hash {
-        local ($_) = @_;
-        eval "use MIME::Base32 qw( RFC ); use Digest::Tiger;";
-        #$_.=("\x00"x(1024 - length $_));print ( 'hlen', length $_);
-        MIME::Base32::encode( Digest::Tiger::hash($_) );
-      }
-      #$self->log('TIGERTEST',  hash('Tiger'));
-      #$self->log('TIGERTESTid',  hash('FXC3WTTDXHP7PLCCGZ6ZKBHRVAKBQ4KUINROXXI'));
-      #$self->log('TIGERTESTpd', hash('P26YAWX3HUNSTEXXYRGOIAAM2ZPMLD44HCWQEDY'));
-      $self->{'PD'} ||= hash( 'perl' . $self->{'myip'} . $self->{'NI'} . time );
-      $self->{'ID'} ||= hash( $self->{'PD'} );
-      $self->{'SL'} ||= $self->{'S'} || '2';
-      $self->{'SS'} ||= $self->{'sharesize'} || 20025693588;
-      $self->{'SF'} ||= 30999;
-      $self->{'HN'} ||= $self->{'H'} || 1;
-      $self->{'HR'} ||= $self->{'R'} || 0;
-      $self->{'HO'} ||= $self->{'O'} || 0;
-      $self->{'VE'} ||= $self->{'V'} || '++\s0.706';
-      $self->{'US'} ||= 10000;
-      $self->{'SU'} ||= 'ADC0';
-      #$self->{''} ||= $self->{''} || '';
-      $self->sendcmd( 'BINF', $self->{'sid'}, map { $_ . $self->{$_} } grep { $self->{$_} } @{ $self->{'BINFS'} } );
-      #BINF UUXX IDFXC3WTTDXHP7PLCCGZ6ZKBHRVAKBQ4KUINROXXI PDP26YAWX3HUNSTEXXYRGOIAAM2ZPMLD44HCWQEDY NI SL2 SS20025693588
-      #SF30999 HN2 HR0 HO0 VE++\s0.706 US5242 SUADC0
-      }
   };
-  if ( $self->{'M'} eq 'A' ) {
+  $self->log( 'dev', "0making listeners [$self->{'M'}]" );
+  if ( $self->{'M'} eq 'A' or !$self->{'M'} ) {
     $self->log( 'dev', "making listeners: tcp" );
     $self->{'clients'}{'listener_tcp'} = $self->{'incomingclass'}->new(
       %$self, $self->clear(),
@@ -392,6 +421,7 @@ sub init {
       'PortList'    => \%{ $self->{'PortList'} },
       'handler'     => \%{ $self->{'handler'} },
       'auto_listen' => 1,
+      'parent'      => $self,
     );
     $self->{'myport'} = $self->{'myport_tcp'} = $self->{'clients'}{'listener_tcp'}{'myport'};
     $self->log( 'err', "cant listen tcp (file transfers)" ) unless $self->{'myport_tcp'};
@@ -409,8 +439,8 @@ sub init {
       #'debug'=>1,
       #'nonblocking' => 0,
       'parse' => {
-        'SR'   => $self->{'parse'}{'SR'},
-        'UPSR' => sub {
+        'SR'  => $self->{'parse'}{'SR'},
+        'PSR' => sub {                     #U
           #$self->log( 'dev', "UPSR", @_ );
         },
 #2008/12/14-13:30:50 [3] rcv: welcome UPSR FQ2DNFEXG72IK6IXALNSMBAGJ5JAYOQXJGCUZ4A NIsss2911 HI81.9.63.68:4111 U40 TRZ34KN23JX2BQC2USOTJLGZNEWGDFB327RRU3VUQ PC4 PI0,64,92,94,100,128,132,135 RI64,65,66,67,68,68,69,70,71,72
@@ -421,6 +451,7 @@ sub init {
 #$SR [Predator]Wolf DC++\Btyan Adams - Please Forgive Me.mp314217310 18/20TTH:G7DXSTGPHTXSD2ZZFQEUBWI7PORILSKD4EENOII (81.9.63.68:4111)
       },
       'auto_listen' => 1,
+      'parent'      => $self,
     );
     $self->{'myport_udp'} = $self->{'clients'}{'listener_udp'}{'myport'};
     $self->log( 'err', "cant listen udp (search repiles)" ) unless $self->{'myport_udp'};
@@ -445,5 +476,9 @@ sub init {
       unless $self->{'myport_http'};
 =cut
 
+  $self->{'handler_int'}{'disconnect_bef'} = sub {
+    delete $self->{'sid'};
+    #$self->log( 'dev', 'disconnect int' ) if $self and $self->{'log'};
+  };
 }
 1;
